@@ -1,12 +1,12 @@
 /*                     __                                               *\
 **     ________ ___   / /  ___     Scala API                            **
-**    / __/ __// _ | / /  / _ |    (c) 2002-2006, LAMP/EPFL             **
-**  __\ \/ /__/ __ |/ /__/ __ |                                         **
+**    / __/ __// _ | / /  / _ |    (c) 2002-2007, LAMP/EPFL             **
+**  __\ \/ /__/ __ |/ /__/ __ |    http://www.scala-lang.org/           **
 ** /____/\___/_/ |_/____/_/ | |                                         **
 **                          |/                                          **
 \*                                                                      */
 
-// $Id: Proxy.scala 12340 2007-07-17 15:29:47Z mcdirmid $
+// $Id: Proxy.scala 13018 2007-10-08 16:09:53Z michelou $
 
 
 package scala
@@ -26,8 +26,8 @@ trait Proxy {
   def self: Any
   override def hashCode: Int = self.hashCode
   override def equals(that: Any): Boolean = that match {
-  case that : Proxy => self.equals(that.self)
-  case that         => self.equals(that)
+    case that: Proxy => self equals that.self
+    case that        => self equals that
   }
   override def toString: String = self.toString
 }
