@@ -1,23 +1,22 @@
 /* NSC -- new Scala compiler
- * Copyright 2005-2006 LAMP/EPFL
+ * Copyright 2005-2007 LAMP/EPFL
  * @author  Martin Odersky
  */
-// $Id: TreeBuilder.scala 11142 2007-05-22 10:23:00Z mcdirmid $
+// $Id: TreeBuilder.scala 12886 2007-09-17 16:36:10Z mcdirmid $
 
 package scala.tools.nsc.ast.parser
-import scala.tools.nsc.util.Position
 
 import symtab.Flags._
 import scala.collection.mutable.ListBuffer
+import scala.tools.nsc.util.Position
 
 abstract class TreeBuilder {
 
   val global: Global
   import global._
   import posAssigner.atPos;
-  def freshName(prefix: String, pos : Position): Name
-
-  def freshName(pos : Position): Name = freshName("x$", pos)
+  def freshName(pos : Position, prefix: String): Name
+  def freshName(pos : Position): Name = freshName(pos, "x$")
 
   def scalaDot(name: Name): Tree =
     Select(Ident(nme.scala_) setSymbol definitions.ScalaPackage, name)
@@ -57,16 +56,16 @@ abstract class TreeBuilder {
 
   /** Traverse pattern and collect all variable names with their types in buffer */ 
   private object getvarTraverser extends Traverser {
-    val buf = new ListBuffer[(Name, Tree)]
+    val buf = new ListBuffer[(Name, Tree, Position)]
     def init: Traverser = { buf.clear; this }
-    override def traverse(tree: Tree): unit = tree match {
+    override def traverse(tree: Tree): Unit = tree match {
       case Bind(name, Typed(tree1, tpt)) =>
         if ((name != nme.WILDCARD) && (buf.elements forall (name !=)))
-          buf += (name, if (treeInfo.mayBeTypePat(tpt)) TypeTree() else tpt)
+          buf += (name, if (treeInfo.mayBeTypePat(tpt)) TypeTree() else tpt, tree.pos)
         traverse(tree1)
       case Bind(name, tree1) =>
         if ((name != nme.WILDCARD) && (buf.elements forall (name !=)))
-          buf += (name, TypeTree())
+          buf += (name, TypeTree(), tree.pos)
         traverse(tree1)
       case _ =>
         super.traverse(tree)
@@ -76,23 +75,23 @@ abstract class TreeBuilder {
   /** Returns list of all pattern variables, possibly with their types,
    *  without duplicates
    */
-  private def getVariables(tree: Tree): List[(Name, Tree)] = {
+  private def getVariables(tree: Tree): List[(Name, Tree,Position)] = {
     getvarTraverser.init.traverse(tree)
     getvarTraverser.buf.toList
   }
 
-  private def makeTuple(trees: List[Tree], isType: boolean): Tree = {
+  private def makeTuple(trees: List[Tree], isType: Boolean): Tree = {
     val tupString = "Tuple" + trees.length
     Apply(scalaDot(if (isType) newTypeName(tupString) else newTermName(tupString)), trees)
   }
 
-  def makeTupleTerm(trees: List[Tree], flattenUnary: boolean): Tree = trees match {
+  def makeTupleTerm(trees: List[Tree], flattenUnary: Boolean): Tree = trees match {
     case List() => Literal(())
     case List(tree) if flattenUnary => tree
     case _ => makeTuple(trees, false)
   }
 
-  def makeTupleType(trees: List[Tree], flattenUnary: boolean): Tree = trees match {
+  def makeTupleType(trees: List[Tree], flattenUnary: Boolean): Tree = trees match {
     case List() => scalaUnitConstr
     case List(tree) if flattenUnary => tree
     case _ => AppliedTypeTree(scalaDot(newTypeName("Tuple" + trees.length)), trees)
@@ -119,7 +118,7 @@ abstract class TreeBuilder {
   }
 
   /** Create tree representing (unencoded) binary operation expression or pattern. */
-  def makeBinop(isExpr: boolean, left: Tree, op: Name, right: Tree): Tree = {
+  def makeBinop(isExpr: Boolean, left: Tree, op: Name, right: Tree): Tree = {
     val arguments = right match {
       case Parens(args) => args
       case _ => List(right)
@@ -140,7 +139,9 @@ abstract class TreeBuilder {
 
   /** Create tree representing an object creation <new parents { stats }> */
   def makeNew(parents: List[Tree], self: ValDef, stats: List[Tree], argss: List[List[Tree]]): Tree =
-    if (parents.tail.isEmpty && stats.isEmpty)
+    if (parents.isEmpty)
+      makeNew(List(scalaAnyRefConstr), self, stats, argss)
+    else if (parents.tail.isEmpty && stats.isEmpty)
       New(parents.head, argss)
     else {
       val x = nme.ANON_CLASS_NAME.toTypeName
@@ -184,7 +185,7 @@ abstract class TreeBuilder {
     else Block(stats.init, stats.last)
 
   /** Create tree for for-comprehension generator &lt;val pat0 &lt;- rhs0&gt; */
-  def makeGenerator(pos: Position, pat: Tree, valeq: boolean, rhs: Tree): Enumerator = {
+  def makeGenerator(pos: Position, pat: Tree, valeq: Boolean, rhs: Tree): Enumerator = {
     val pat1 = patvarTransformer.transform(pat);
     val rhs1 =
       if (valeq) rhs
@@ -211,6 +212,9 @@ abstract class TreeBuilder {
   def makeSyntheticParam(pname: Name) = 
     ValDef(Modifiers(PARAM | SYNTHETIC), pname, TypeTree(), EmptyTree)
 
+  def makeSyntheticTypeParam(pname: Name, bounds: Tree) = 
+    TypeDef(Modifiers(DEFERRED | SYNTHETIC), pname, List(), bounds)
+
   abstract class Enumerator
   case class ValFrom(pos: Position, pat: Tree, rhs: Tree) extends Enumerator
   case class ValEq(pos: Position, pat: Tree, rhs: Tree) extends Enumerator
@@ -300,7 +304,7 @@ abstract class TreeBuilder {
                 ValFrom(pos, pat, makeCombination(nme.filter, rhs, pat.duplicate, test)) :: rest,
                 body)
       case ValFrom(pos, pat, rhs) :: rest =>
-        val valeqs = rest.take(definitions.MaxTupleArity - 1).takeWhile(.isInstanceOf[ValEq]);
+        val valeqs = rest.take(definitions.MaxTupleArity - 1).takeWhile(_.isInstanceOf[ValEq]);
         assert(!valeqs.isEmpty)
         val rest1 = rest.drop(valeqs.length)
         val pats = valeqs map { case ValEq(_, pat, _) => pat }
@@ -365,15 +369,15 @@ abstract class TreeBuilder {
     makeAlternative(List(p, Sequence(List())))
 
   /** Create visitor <x => x match cases> */
-  def makeVisitor(cases: List[CaseDef], checkExhaustive: boolean): Tree =
+  def makeVisitor(cases: List[CaseDef], checkExhaustive: Boolean): Tree =
     makeVisitor(cases, checkExhaustive, "x$")
 
   private def makeUnchecked(expr: Tree): Tree =
     Annotated(Annotation(New(scalaDot(definitions.UncheckedClass.name), List(List())), List()), expr)
 
   /** Create visitor <x => x match cases> */
-  def makeVisitor(cases: List[CaseDef], checkExhaustive: boolean, prefix: String): Tree = {
-    val x = freshName(prefix, posAssigner.pos)
+  def makeVisitor(cases: List[CaseDef], checkExhaustive: Boolean, prefix: String): Tree = {
+    val x = freshName(posAssigner.pos, prefix)
     val sel = if (checkExhaustive) Ident(x) else makeUnchecked(Ident(x))
     Function(List(makeSyntheticParam(x)), Match(sel, cases))
   }
@@ -389,37 +393,38 @@ abstract class TreeBuilder {
   /** Create tree for pattern definition &lt;mods val pat0 = rhs&gt; */
   def makePatDef(mods: Modifiers, pat: Tree, rhs: Tree): List[Tree] = matchVarPattern(pat) match {
     case Some((name, tpt)) =>
-      List(ValDef(mods, name, tpt, rhs))
+      List(ValDef(mods, name, tpt, rhs).setPos(pat.pos))
 
     case None =>
       //  in case there are no variables in pattern
-      //  val p = e  ==>  e.match (case p => ())
+      //  val/var p = e  ==>  e.match (case p => ())
       //
-      //  in case there is exactly one variable in pattern
-      //  val x_1 = e.match (case p => (x_1))
+      //  in case there is exactly one variable x_1 in pattern
+      //  val/var p = e  ==>  val/var x_1 = e.match (case p => (x_1))
       //
       //  in case there are more variables in pattern
-      //  val p = e  ==>  private synthetic val t$ = e.match (case p => (x_1, ..., x_N))
-      //                  val x_1 = t$._1
+      //  val/var p = e  ==>  private synthetic val t$ = e.match (case p => (x_1, ..., x_N))
+      //                  val/var x_1 = t$._1
       //                  ...
-      //                  val x_N = t$._N
+      //                  val/var x_N = t$._N
       val pat1 = patvarTransformer.transform(pat)
       val vars = getVariables(pat1)
       val matchExpr = atPos(pat1.pos){
         Match(
           makeUnchecked(rhs), 
-          List(CaseDef(pat1, EmptyTree, makeTupleTerm(vars map (._1) map Ident, true))))
+          List(CaseDef(pat1, EmptyTree, makeTupleTerm(vars map (_._1) map Ident, true))))
       }
       vars match {
         case List() => 
           List(matchExpr)
-        case List((vname, tpt)) =>
-          List(ValDef(mods, vname, tpt, matchExpr))
+        case List((vname, tpt, pos)) =>
+          List(ValDef(mods, vname, tpt, matchExpr).setPos(pos))
         case _ => 
           val tmp = freshName(pat1.pos)
-          val firstDef = ValDef(Modifiers(PRIVATE | LOCAL | SYNTHETIC), tmp, TypeTree(), matchExpr)
+          val firstDef = ValDef(Modifiers(PRIVATE | LOCAL | SYNTHETIC | (mods.flags & LAZY)), 
+                                tmp, TypeTree(), matchExpr)
           var cnt = 0
-          val restDefs = for (val (vname, tpt) <- vars) yield {
+          val restDefs = for (val (vname, tpt, pos) <- vars) yield atPos(pos) {
             cnt = cnt + 1
             ValDef(mods, vname, tpt, Select(Ident(tmp), newTermName("_" + cnt)))
           }
@@ -432,13 +437,15 @@ abstract class TreeBuilder {
     AppliedTypeTree(scalaDot(newTypeName("Function" + argtpes.length)), argtpes ::: List(restpe))
 
   /** Create a tree representing a the name function type (=> argtpe) => restpe */
+  /*
   def makeByNameFunctionTypeTree(argtpe: Tree, restpe: Tree): Tree = 
     AppliedTypeTree(scalaDot(newTypeName("ByNameFunction")), List(argtpe, restpe))
+  */
 
   /** Append implicit view section if for `implicitViews' if nonempty */
   def addImplicitViews(owner: Name, vparamss: List[List[ValDef]], implicitViews: List[Tree]): List[List[ValDef]] = {
     val mods = Modifiers(if (owner.isTypeName) PARAMACCESSOR | LOCAL | PRIVATE else PARAM)
-    def makeViewParam(tpt: Tree) = ValDef(mods | IMPLICIT, freshName("view$", tpt.pos), tpt, EmptyTree)
+    def makeViewParam(tpt: Tree) = ValDef(mods | IMPLICIT, freshName(tpt.pos, "view$"), tpt, EmptyTree)
     if (implicitViews.isEmpty) vparamss
     else vparamss ::: List(implicitViews map makeViewParam)
   }
@@ -446,9 +453,9 @@ abstract class TreeBuilder {
   /** Create a tree representing a packaging */
   def makePackaging(pkg: Tree, stats: List[Tree]): PackageDef = pkg match {
     case Ident(name) =>
-      PackageDef(name, stats)
+      PackageDef(name, stats).setPos(pkg.pos)
     case Select(qual, name) =>
-      makePackaging(qual, List(PackageDef(name, stats)))
+      makePackaging(qual, List(PackageDef(name, stats).setPos(pkg.pos)))
   }
 
   case class Parens(args: List[Tree]) extends Tree
